@@ -1,6 +1,6 @@
 package Parse::Dia::SQL;
 
-# $Id: SQL.pm,v 1.29 2009/05/16 18:02:51 aff Exp $
+# $Id: SQL.pm,v 1.31 2009/06/23 19:54:29 aff Exp $
 
 =pod
 
@@ -45,6 +45,14 @@ See L<http://tedia2sql.tigris.org/usingtedia2sql.html>
 
 =back
 
+=head1 DIA VERSIONS
+
+Parse::Dia::SQL has been tested with Dia versions 0.93 - 0.97.  
+
+Dia version 0.97 changed the way associations were stored, and support for this has been added as of Parse::Dia::SQL version 0.11.
+
+Parse::Dia::SQL uses the XML C<version> tag information in the I<.dia> input file to determine how each XML construct is formatted.  Future versions of Dia may change the internal format, and XML C<version> tag is used to detect such changes.
+
 =head1 DATABASE SUPPORT
 
 The following databases are supported:
@@ -180,7 +188,12 @@ use Parse::Dia::SQL::Output::Sas;
 use Parse::Dia::SQL::Output::Sybase;
 use Parse::Dia::SQL::Output::SQLite3;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
+
+my $UML_ASSOCIATION  = 'UML - Association';
+my $UML_SMALLPACKAGE = 'UML - SmallPackage';
+my $UML_CLASS        = 'UML - Class';
+my $UML_COMPONENT    = 'UML - Component';
 
 =head1 METHODS
 
@@ -298,7 +311,7 @@ sub get_output_instance {
       qw(classes associations small_packages components files index_options);
 
   if ($self->{db} eq q{db2}) {
-	return Parse::Dia::SQL::Output::DB2->new(%param);
+  return Parse::Dia::SQL::Output::DB2->new(%param);
   } elsif ($self->{db} eq q{mysql-myisam}) {
     return Parse::Dia::SQL::Output::MySQL::MyISAM->new(%param);
   } elsif ($self->{db} eq q{mysql-innodb}) {
@@ -450,17 +463,25 @@ sub _parse_smallpackages {
 
     $self->{log}->debug( "nodelist length" . $nodelist->getLength );
 
+	NODE:
     for ( my $i = 0 ; $i < $nodelist->getLength ; $i++ ) {
       my $nodeType = $nodelist->item($i)->getNodeType;
 
       # sanity check -- a dia:object should be an element_node
       if ( $nodeType == ELEMENT_NODE ) {
-        my $nodeAttrType = $nodelist->item($i)->getAttribute('type');
-        my $nodeAttrId   = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrType    = $nodelist->item($i)->getAttribute('type');
+        my $nodeAttrId      = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrVersion = $nodelist->item($i)->getAttribute('version');
         $self->{log}
           ->debug("Node $i -- type=$nodeAttrType");
 
-        if ( $nodeAttrType eq 'UML - SmallPackage' ) {
+        if ( $nodeAttrType eq $UML_SMALLPACKAGE ) {
+
+					# Check that version is supported
+					if (!$self->{utils}->_check_object_version($UML_SMALLPACKAGE, $nodeAttrVersion)) {
+						$self->{log}->error("Found unsupported version '$nodeAttrVersion' of $UML_SMALLPACKAGE");
+						next NODE;
+					}
 
           # generic database statements
           $self->{log}->debug("call generateSmallPackageSQL");
@@ -497,8 +518,8 @@ sub _parse_smallpackage {
   my $packName = undef;
   for ( my $i = 0 ; $i < $nodelist->getLength ; $i++ ) {
     my $currentNode  = $nodelist->item($i);
-    my $nodeAttrName = $currentNode->getAttribute('name');
-    $self->{log}->debug("nodeAttrName:$nodeAttrName");
+    my $nodeAttrName    = $currentNode->getAttribute('name');
+    $self->{log}->debug("nodeAttrName   :$nodeAttrName");
 
     if ( $nodeAttrName eq 'stereotype' ) {
       $packName = $self->{utils}->get_string_from_node($currentNode);
@@ -525,6 +546,7 @@ sub get_classes_ref {
 }
 
 
+
 # Returns hashref where key is name of class and value is its content.
 sub _parse_classes {
   my $self    = shift;
@@ -540,29 +562,46 @@ sub _parse_classes {
     $self->{log}
       ->debug("nodelist length " . $nodelist->getLength );
 
+	NODE:
     for ( my $i = 0 ; $nodelist && $i < $nodelist->getLength ; $i++ ) {
       my $nodeType = $nodelist->item($i)->getNodeType;
 
       # sanity check -- a dia:object should be an element_node
       if ( $nodeType == ELEMENT_NODE ) {
-        my $nodeAttrType = $nodelist->item($i)->getAttribute('type');
-        my $nodeAttrId   = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrType    = $nodelist->item($i)->getAttribute('type');
+        my $nodeAttrId      = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrVersion = $nodelist->item($i)->getAttribute('version');
 
         $self->{log}->debug("Node $i -- type=$nodeAttrType");
 
-        if ( $nodeAttrType eq 'UML - Class' ) {
+        if ( $nodeAttrType eq $UML_CLASS ) {
+
+					# Check that version is supported
+					if (!$self->{utils}->_check_object_version($UML_CLASS, $nodeAttrVersion)) {
+						$self->{log}->error("Found unsupported version '$nodeAttrVersion' of UML Class");
+						next NODE;
+					}
+
 
           # table or view create
           $self->{log}->debug("$nodeAttrId");
-          my $class = $self->_parse_class( $nodelist->item($i), [$fid, $nodeAttrId] );
+          my $class = $self->_parse_class( $nodelist->item($i), [$fid, $nodeAttrId, $nodeAttrVersion] );
+
           push @{$self->{classes}}, $class;
 
           #$self->{log}->debug("get_class:". Dumper($class));
 
         }
-        elsif ( $nodeAttrType eq 'UML - Component' ) {
+        elsif ( $nodeAttrType eq $UML_COMPONENT ) {
           #$self->{log}->debug("get_component");
 
+					# Check that version is supported
+					if (!$self->{utils}->_check_object_version($UML_COMPONENT, $nodeAttrVersion)) {
+						$self->{log}->error("Found unsupported version '$nodeAttrVersion' of $UML_COMPONENT");
+						next NODE;
+					}
+
+
           # insert statements - hash ref where table is key
           my $component = $self->_parse_component ($nodelist->item($i), [$i, $nodeAttrId]);
           push @{$self->{components}}, $component if defined($component);
@@ -888,14 +927,18 @@ sub _parse_associations {
 
       # sanity check -- a dia:object should be an element_node
       if ( $nodeType == ELEMENT_NODE ) {
-        my $nodeAttrType = $nodelist->item($i)->getAttribute('type');
-        my $nodeAttrId   = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrType    = $nodelist->item($i)->getAttribute('type');
+        my $nodeAttrId      = $nodelist->item($i)->getAttribute('id');
+        my $nodeAttrVersion = $nodelist->item($i)->getAttribute('version');
 
+        if ( $nodeAttrType eq $UML_ASSOCIATION ) {
+          $self->{log}->debug("Association Node $i -- type=$nodeAttrType id=$nodeAttrId version=$nodeAttrVersion");
 
-        if ( $nodeAttrType eq 'UML - Association' ) {
-        $self->{log}->debug("Association Node $i -- type=$nodeAttrType id=$nodeAttrId");
-        # TODO: Check return value:
-        $self->_parse_association( $nodelist->item($i), [ $fid, $nodeAttrId ] )
+          # Note that version number is passed since there was a
+          # change in Dia 0.97
+
+          # TODO: Check return value:
+          $self->_parse_association( $nodelist->item($i), [ $fid, $nodeAttrId, $nodeAttrVersion ] )
         }
       }
 
@@ -908,6 +951,23 @@ sub _parse_associations {
 # Generate the foreign key relationship between two tables: classify
 # the relationship, and generate the necessary constraints and centre
 # (join) tables.
+#
+# Note that version number is passed as an argument (in '$id') since
+# there was a change in Dia 0.97.  This is implemented in dia source
+# file:
+#
+# objects/UML/association.c
+#
+#   /* Version 0 had no autorouting and so shouldn't have it set by default. */
+#   /* Version 1 was saving both ends separately without using StdProps */
+#   /* Version 2 uses StdProps */
+#
+# Note on misspelling of "multipicity"
+#  
+# http://mail.gnome.org/archives/dia-list/2009-March/msg00067.html
+# [Hans Breuer] "Sorry, typos in property names must stay forever to
+# not break backward compatibility with older diagrams."
+
 sub _parse_association {
   my $self        = shift;
   my $association = shift;
@@ -918,7 +978,14 @@ sub _parse_association {
   my ( %leftEnd, %rightEnd, $connectionNode, $leftConnectionHandle,
     $rightConnectionHandle );
 
-  $self->{log}->debug("Parsing UML Association file=[$id->[0]] id=$id->[1]");
+  my $version = $id->[2];
+  $self->{log}->debug("Parsing UML Association file=$id->[0] id=$id->[1] version=$version");
+
+  # Check that version is supported
+  if (!$self->{utils}->_check_object_version($UML_ASSOCIATION, $version)) {
+    $self->{log}->error("Found unsupported version '$version' of $UML_ASSOCIATION");
+    return;
+  }
 
   $nodeList = $association->getElementsByTagName('dia:attribute');
 
@@ -929,26 +996,56 @@ sub _parse_association {
     $currentNode  = $nodeList->item($i);
     $nodeAttrName = $currentNode->getAttribute('name');
 
-    if ( $nodeAttrName eq 'name' ) {
-      $assocName = $self->{utils}->get_string_from_node($currentNode);
-      $self->{log}->debug("Got association name=$assocName");
-    }
-    elsif ( $nodeAttrName eq 'direction' ) {
-      $assocDirection = $self->{utils}->get_num_from_node($currentNode);
-    }
-    elsif ( $nodeAttrName eq 'ends' ) {
+    $self->{log}->debug( "nodeAttrName:$nodeAttrName" );
 
-      # cycle through dia:composite entries looking for string role &
-      # numeric aggregate values get the attributes for this
-      # association -- each is a dia:composite
-      #
-      # there should only be one dia:composite within the association
+    # version 1 : Dia 0.96 or prior
+    if ($version == 1) {
 
-      my @tags = ( 'arole', '9aggregate', 'bclass_scope', 'amultiplicity' );
-      %leftEnd = $self->{utils}->get_node_attribute_values(
-        $association->getElementsByTagName('dia:composite')->item(0), @tags );
-      %rightEnd = $self->{utils}->get_node_attribute_values(
-        $association->getElementsByTagName('dia:composite')->item(1), @tags );
+      if ( $nodeAttrName eq 'name' ) {
+        $assocName = $self->{utils}->get_string_from_node($currentNode);
+        $self->{log}->debug("Got association name=$assocName");
+      } elsif ( $nodeAttrName eq 'direction' ) {
+        $assocDirection = $self->{utils}->get_num_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'ends' ) {
+        my @tags = ( 'arole', '9aggregate', 'bclass_scope', 'amultiplicity' );
+        %leftEnd = $self->{utils}->get_node_attribute_values(
+                                                             $association->getElementsByTagName('dia:composite')->item(0), @tags );
+        %rightEnd = $self->{utils}->get_node_attribute_values(
+                                                              $association->getElementsByTagName('dia:composite')->item(1), @tags );
+
+      } 
+    }
+
+    # version 2 : Dia 0.97 or later - Note (mis)spelling of 'multipicity':
+    elsif ( $version == 2) {
+
+      if ( $nodeAttrName eq 'name' ) {
+        $assocName = $self->{utils}->get_string_from_node($currentNode);
+        $self->{log}->debug("Got association name=$assocName");
+      } elsif ( $nodeAttrName eq 'direction' ) {
+        $assocDirection = $self->{utils}->get_num_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'role_a' ) {
+        $leftEnd{role} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'role_b' ) {
+        $rightEnd{role} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'aggregate_a' ) {
+        $leftEnd{aggregate} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'aggregate_b' ) {
+        $rightEnd{aggregate} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'class_scope_a' ) {
+        $leftEnd{class_scope} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName eq 'class_scope_b' ) {
+        $rightEnd{class_scope} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName =~ qr/^multip[l]icity_a$/ ) { ### Spelling !!!
+        # Accept both 'multipicity_a'and 'multiplicity_a' should it be fixed upstream
+        $leftEnd{multiplicity} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $nodeAttrName =~ qr/^multip[l]icity_a$/ ) { ### Spelling !!!
+        # Accept both 'multipicity_b'and 'multiplicity_b' should it be fixed upstream
+        $rightEnd{multiplicity} = $self->{utils}->get_string_from_node($currentNode);
+      }
+
+    } else {
+      $self->{log}->fatal("Unsupported $UML_ASSOCIATION version $version");
     }
 
     $i++;
@@ -988,18 +1085,23 @@ sub _parse_association {
     return;
   }
 
-  $self->{log}->debug(
-    "  * (UNUSED) direction=$assocDirection (aggregate determines many end)");
-  $self->{log}->debug( "  * leftEnd="
-      . $leftEnd{'role'} . " agg="
-      . $leftEnd{'aggregate'}
-      . " classId="
-      . $leftConnectionHandle );
-  $self->{log}->debug( "  * rightEnd="
-      . $rightEnd{'role'} . " agg="
-      . $rightEnd{'aggregate'}
-      . " classId="
-      . $rightConnectionHandle );
+  if ($self->{log}->is_debug()) {
+    $self->{log}->debug("leftEnd : ".Dumper(\%leftEnd));
+    $self->{log}->debug("rightEnd: ".Dumper(\%rightEnd));
+
+    $self->{log}->debug(
+                        "  * (UNUSED) direction=$assocDirection (aggregate determines many end)");
+    $self->{log}->debug( "  * leftEnd="
+                         . $leftEnd{'role'} . " agg="
+                         . $leftEnd{'aggregate'}
+                         . " classId="
+                         . $leftConnectionHandle );
+    $self->{log}->debug( "  * rightEnd="
+                         . $rightEnd{'role'} . " agg="
+                         . $rightEnd{'aggregate'}
+                         . " classId="
+                         . $rightConnectionHandle );
+  }
 
   my $leftMult  = $self->{utils}->classify_multiplicity( $leftEnd{'multiplicity'} );
   my $rightMult = $self->{utils}->classify_multiplicity( $rightEnd{'multiplicity'} );
@@ -1521,7 +1623,7 @@ sub generate_one_to_any_association {
   else {
     
     $self->{log}->warn( "No FK attibute in specified in $assocName");  
-	# TODO: Implement the below method:
+  # TODO: Implement the below method:
     #$fkEndKey = fkNamesFromAttList( $pkClassLookup->{name}, $pkAtts );
   }
   $fkAtts = $self->{utils}->attlist_from_names( $fkClassLookup, $fkEndKey );
