@@ -1,6 +1,6 @@
 package Parse::Dia::SQL::Utils;
 
-# $Id: Utils.pm,v 1.5 2009/04/03 05:35:17 aff Exp $
+# $Id: Utils.pm,v 1.7 2009/06/23 19:54:29 aff Exp $
 
 =pod
 
@@ -126,14 +126,9 @@ sub get_node_attribute_values {
   my ( $currentNode, $nodeAttrName, $i );
   my %return;
 
- # drTAE: hopefully Dia will never use this string as an actual value in its XML
- # document save format, 'cuz I'm using it as a special "undef/NULL" construct
-  my $emptyValueString =
-"__this_is_a_very_very_empty_STRING_value_that_i_hope_no-one_will_ever_use (it's like NULL or 'undef')__";
-  my $emptyValueNumber =
-"__this_is_a_very_very_empty_NUMERIC_value_that_i_hope_no-one_will_ever_use (it's like NULL or 'undef')__";
-  my $emptyValueBoolean =
-"__this_is_a_very_very_empty_BOOLEAN_value_that_i_hope_no-one_will_ever_use (it's like NULL or 'undef')__";
+  my $emptyValueString  = "__undef_string__";
+  my $emptyValueNumber  = "__undef_numeric__";
+  my $emptyValueBoolean = "__undef_boolean__";
 
   # initialise it to a bunch of empty values, this will also allow
   # us to know which attribute name values to parse out of the
@@ -173,6 +168,11 @@ sub get_node_attribute_values {
     }
   }
 
+	{
+		no warnings q{uninitialized};
+		$self->{log}->debug( "$nodeAttrName:" . $return{$nodeAttrName} );
+	} 
+
   # don't return some fake value for bits we didn't parse,
   # return undef which means it wasn't there
   foreach my $singleInfo (@infosToGet) {
@@ -359,6 +359,7 @@ Look at a multiplicity descriptor and classify it as 'one' (1, or
 sub classify_multiplicity {
   my $self    = shift;
   my $multStr = shift;
+  return 'none'  if ( ! $multStr );
   $multStr =~ s/\s//g;
   my @mult = split( /\.\./, $multStr );
   return 'none'  if ( @mult == 0 );
@@ -523,7 +524,7 @@ Check that a list of primary key attributes has types corresponding to
 the types in a list of foreign key attributes.
 
 Returns base type of some DMBS specific types (eg in PostgreSQL serial
-is int).
+is integer).
 
 =cut
 
@@ -534,8 +535,8 @@ sub get_base_type {
   if ( $db eq 'postgres' ) {
 
     # handle PostgreSQL database type
-    if ( $typeName eq 'serial' || $typeName eq 'SERIAL' ) {
-      return 'int';
+    if ( lc($typeName) eq 'serial' ) {
+      return 'integer';
     }
     return $typeName;
   }
@@ -842,6 +843,42 @@ sub add_default_fk {
 }
 
 
+# Check that the given object and version is supported.  Return true
+# on pass, undef on fail.
+sub _check_object_version {
+  my $self    = shift;
+  my $type    = shift;
+  my $version = int shift;    # can be zero, can have leading zeros
+  
+  if (!$type || !defined $version) {
+    $self->{log}->error(qq{Need 2 args: type and version});
+    return;
+  }
+
+  my %object_v = (
+                  "UML - Association"  => [1,2],
+                  "UML - Class"        => [0],
+                  "UML - Component"    => [0],
+                  "UML - Note"         => [0],
+                  "UML - SmallPackage" => [0],
+                 );
+
+  $self->{log}->debug(qq{type:'$type' version:$version});
+
+  if (!exists($object_v{$type})) {
+    $self->{log}->debug(qq{type:'$type' unknown});
+    return;
+  }
+
+  if (! grep(/^$version$/, @{$object_v{$type}})) {
+    $self->{log}->debug(qq{type:'$type' version:$version unsupported});
+    return;
+  }
+
+  return 1;
+}
+
+
 # =head2 parseExtras
 
 # Parse the name of a Small Package that contains extra SQL clauses for
